#include "kernelsolver.h"

const double alpha=1.0;
const double beta=0.0;

float solve_cublas(double *mA, double *mB, double *Result, int n){
  double *dev_mA;
  double *dev_mB;
  double *dev_Result;

  /*выделение памяти*/
  cudaMalloc((void**)&dev_mA,n*n*sizeof(*mA));
  cudaMalloc((void**)&dev_mB,n*n*sizeof(*mB));
  cudaMalloc((void**)&dev_Result,n*n*sizeof(*Result));

  cublasHandle_t handle;
  cublasCreate(&handle);

  /*копирование в память gpu*/
  cublasSetMatrix(n,n,sizeof(*mA),mA,n,dev_mA,n);
  cublasSetMatrix(n,n,sizeof(*mB),mB,n,dev_mB,n);

  cudaEvent_t start,stop;
  float gputime=0;

  cudaEventCreate(&start);
  cudaEventCreate(&stop);

  cudaEventRecord(start,0);

  /*произведение матрицы на вектор*/
  cublasDgemm(handle,CUBLAS_OP_T,CUBLAS_OP_N,
		n,n,n,
		&alpha,
		dev_mA,n,
		dev_mB,n,
		&beta,
		dev_Result,n);

  cudaEventRecord(stop,0);
  cudaEventSynchronize(stop);
  cudaEventElapsedTime(&gputime,start,stop);

  /*копирование в память cpu*/
  cublasGetMatrix(n,n,sizeof(*Result),dev_Result,n,Result,n);

  /*очистка памяти*/
  cublasDestroy(handle);
  cudaFree(dev_mA);
  cudaFree(dev_mB);
  cudaFree(dev_Result);
  cudaDeviceReset();

  return gputime;
}
